[HACKERS] new Configuration patch, implements 'include'

2003-02-17 Thread mlw
This is a patch that allows PostgreSQL to use a configuration
file that is outside the main database directory.

It adds one more command line parameter, -C which
specifies the location of the postgres configuration file.

A patched version of PostgreSQL will function as:

postmaster -C /etc/postgres/postgresql.conf

This will direct the postmaster program to use the
configuration file /etc/postgres/postgresql.conf

Within this file are four  additional parameters: include,
hba_conf,ident_conf, and data_dir.

They are used as:
include = '/etc/postgres/debug.conf'
data_dir = '/vol01/postgres'
hba_conf = '/etc/postgres/pg_hba_conf'
ident_conf = '/etc/postgres/pg_ident.conf'

The -D option on the command line overrides the data_dir
in the configuration file.

If no hba_conf and/or ident_conf setting is specified, the default
$PGDATA/pg_hba.conf and/or $PGDATA/pg_ident.conf will be used.

This patch is intended to move the PostgreSQL configuration out of the
data directory so that it can be modified and backed up.

This patch is also useful for running multiple servers with the same
parameters:

postmaster -C /etc/postgres/postgresql.conf -D /VOL01/postgres -p 5432
postmaster -C /etc/postgres/postgresql.conf -D /VOL02/postgres -p 5433

To apply the patch, enter your PostreSQL source directory, and run:

cat pgec-PGVERSON.patch | patch -p 1

diff -u -r postgresql-7.3.2/src/backend/libpq/hba.c 
postgresql-7.3.2.ec/src/backend/libpq/hba.c
--- postgresql-7.3.2/src/backend/libpq/hba.cSat Dec 14 13:49:43 2002
+++ postgresql-7.3.2.ec/src/backend/libpq/hba.c Mon Feb 17 09:30:15 2003
@@ -35,6 +35,7 @@
 #include miscadmin.h
 #include nodes/pg_list.h
 #include storage/fd.h
+#include utils/guc.h
 
 
 #define IDENT_USERNAME_MAX 512
@@ -837,10 +838,20 @@
if (hba_lines)
free_lines(hba_lines);
 
-   /* Put together the full pathname to the config file. */
-   bufsize = (strlen(DataDir) + strlen(CONF_FILE) + 2) * sizeof(char);
-   conf_file = (char *) palloc(bufsize);
-   snprintf(conf_file, bufsize, %s/%s, DataDir, CONF_FILE);
+   /* Explicit HBA in config file */
+   if(explicit_hbafile  strlen(explicit_hbafile))
+   {
+   bufsize = strlen(explicit_hbafile)+1;
+   conf_file = (char *) palloc(bufsize);
+   strcpy(conf_file, explicit_hbafile);
+   }
+   else
+   {
+   /* put together the full pathname to the config file */
+   bufsize = (strlen(DataDir) + strlen(CONF_FILE) + 2) * sizeof(char);
+   conf_file = (char *) palloc(bufsize);
+   snprintf(conf_file, bufsize, %s/%s, DataDir, CONF_FILE);
+   }
 
file = AllocateFile(conf_file, r);
if (file == NULL)
@@ -979,10 +990,20 @@
if (ident_lines)
free_lines(ident_lines);
 
-   /* put together the full pathname to the map file */
-   bufsize = (strlen(DataDir) + strlen(USERMAP_FILE) + 2) * sizeof(char);
-   map_file = (char *) palloc(bufsize);
-   snprintf(map_file, bufsize, %s/%s, DataDir, USERMAP_FILE);
+   /* Explicit IDENT in config file */
+   if(explicit_identfile  strlen(explicit_identfile))
+   {
+   bufsize = strlen(explicit_identfile)+1;
+   map_file = (char *) palloc(bufsize);
+   strcpy(map_file, explicit_identfile);
+   }
+   else
+   {
+   /* put together the full pathname to the map file */
+   bufsize = (strlen(DataDir) + strlen(USERMAP_FILE) + 2) * sizeof(char);
+   map_file = (char *) palloc(bufsize);
+   snprintf(map_file, bufsize, %s/%s, DataDir, USERMAP_FILE);
+   }
 
file = AllocateFile(map_file, r);
if (file == NULL)
diff -u -r postgresql-7.3.2/src/backend/postmaster/postmaster.c 
postgresql-7.3.2.ec/src/backend/postmaster/postmaster.c
--- postgresql-7.3.2/src/backend/postmaster/postmaster.cWed Jan 15 19:27:17 
2003
+++ postgresql-7.3.2.ec/src/backend/postmaster/postmaster.c Mon Feb 17 09:30:15 
+2003
@@ -421,7 +421,7 @@
 
opterr = 1;
 
-   while ((opt = getopt(argc, argv, A:a:B:b:c:D:d:Fh:ik:lm:MN:no:p:Ss-:)) != -1)
+   while ((opt = getopt(argc, argv, A:a:B:b:C:c:D:d:Fh:ik:lm:MN:no:p:Ss-:)) != 
+-1)
{
switch (opt)
{
@@ -441,6 +441,9 @@
case 'b':
/* Can no longer set the backend executable file to 
use. */
break;
+   case 'C': // MLW
+   explicit_pgconfig = optarg;
+   break;
case 'D':
potential_DataDir = optarg;
break;
@@ -564,13 +567,23 @@
ExitPostmaster(1);
}
 
-   /*
-* Now we can set the data directory, and then read postgresql.conf.
-*/
-   

Re: [HACKERS] new Configuration patch, implements 'include'

2003-02-17 Thread Tom Lane
mlw [EMAIL PROTECTED] writes:
 If no hba_conf and/or ident_conf setting is specified, the default
 $PGDATA/pg_hba.conf and/or $PGDATA/pg_ident.conf will be used.

Doesn't anybody see the (a) inconsistency and (b) uselessness of this?
If you are trying to keep your config files out of the data directory,
it's hardly sensible to default to finding two out of three there.

We should have a -C that specifies a *directory*, and all three config
files should be sought therein.  The argument that that somehow forces
people to use symlinks doesn't convince me at all.

But I've grown tired of arguing, because it's clear that I'm making no
impact whatever :-(.  I'm done with this thread.

regards, tom lane

---(end of broadcast)---
TIP 6: Have you searched our list archives?

http://archives.postgresql.org



Re: [HACKERS] new Configuration patch, implements 'include'

2003-02-17 Thread mlw
Tom Lane wrote:


mlw [EMAIL PROTECTED] writes:
 

If no hba_conf and/or ident_conf setting is specified, the default
$PGDATA/pg_hba.conf and/or $PGDATA/pg_ident.conf will be used.
   


Doesn't anybody see the (a) inconsistency and (b) uselessness of this?
If you are trying to keep your config files out of the data directory,
it's hardly sensible to default to finding two out of three there.

We should have a -C that specifies a *directory*, and all three config
files should be sought therein.  The argument that that somehow forces
people to use symlinks doesn't convince me at all.

But I've grown tired of arguing, because it's clear that I'm making no
impact whatever :-(.  I'm done with this thread.


Tom, I don't know why you are arguing at all. One thing I wish to 
impress on you, I think it is a point of view you are missing. It isn't 
about something being easier as much as it is about being flexable 
enough to fit into the deployment strategy of the admin or vendor.

Sometimes you make things more difficult when you make it more 
standardized.  When I setup a system with Apache, PHP, PostgreSQL, 
named, et al, I am always just irritated that PostgreSQL's configuration 
parameters can not be stored with all the others. I usually make one 
install tarball or zip that contains all the binaries and configuration. 
I can't do that with PostgreSQL.

I don't like the idea of specifying a directory, per se' because if you 
have multiple database installations, how would you share the 
configuration without symlinks?

I will modify my patch to check if the configuration parameter is a 
directory. If it is, it will make the default filenames within the 
directory and post it when it is gone.


---(end of broadcast)---
TIP 6: Have you searched our list archives?

http://archives.postgresql.org


Re: [HACKERS] new Configuration patch, implements 'include'

2003-02-17 Thread Bruce Momjian
Tom Lane wrote:
 mlw [EMAIL PROTECTED] writes:
  If no hba_conf and/or ident_conf setting is specified, the default
  $PGDATA/pg_hba.conf and/or $PGDATA/pg_ident.conf will be used.
 
 Doesn't anybody see the (a) inconsistency and (b) uselessness of this?
 If you are trying to keep your config files out of the data directory,
 it's hardly sensible to default to finding two out of three there.
 
 We should have a -C that specifies a *directory*, and all three config
 files should be sought therein.  The argument that that somehow forces
 people to use symlinks doesn't convince me at all.

I think the issue here is that symlinks are OK to implement unusual
configuration cases, and I think we can say having the three config
files in different directories is unusual.  I think you have to weigh
the downside of using symlinks for rare configurations compared to the
complexity of specifying the config file locations in three separate
cases.

I had thrown out the idea of putting the config files in their own
directory _under_ /data, like /data/etc, so you could just symlink that
directory to somewhere else.  Makes backups of config files easy, and
makes it initdb-safe, because only the symlink can be under /data. 
However, no one commented on it, so I assume they didn't like it.  It
seems like a nice middle ground to me.

-- 
  Bruce Momjian|  http://candle.pha.pa.us
  [EMAIL PROTECTED]   |  (610) 359-1001
  +  If your life is a hard drive, |  13 Roberts Road
  +  Christ can be your backup.|  Newtown Square, Pennsylvania 19073

---(end of broadcast)---
TIP 2: you can get off all lists at once with the unregister command
(send unregister YourEmailAddressHere to [EMAIL PROTECTED])



Re: [HACKERS] new Configuration patch, implements 'include'

2003-02-17 Thread Bruce Momjian
mlw wrote:
 I don't like the idea of specifying a directory, per se' because if you 
 have multiple database installations, how would you share the 
 configuration without symlinks?

Oh, for example, you would be sharing postgresql.conf, perhaps, but not
pg_hba.conf.

-- 
  Bruce Momjian|  http://candle.pha.pa.us
  [EMAIL PROTECTED]   |  (610) 359-1001
  +  If your life is a hard drive, |  13 Roberts Road
  +  Christ can be your backup.|  Newtown Square, Pennsylvania 19073

---(end of broadcast)---
TIP 6: Have you searched our list archives?

http://archives.postgresql.org